@@ -32,6 +32,7 @@ import luigi.hdfs
 import configuration
 import warnings
 import mrrunner
+import json
 
 logger = logging.getLogger('luigi-interface')
 
@@ -93,7 +94,7 @@ def create_packages_archive(packages, filename):
         if hasattr(package, "__path__"):
             p = package.__path__[0]
 
-            if p.find('.egg') != -1:
+            if p.endswith('.egg') and os.path.isfile(p):
                 raise 'egg files not supported!!!'
                 # Add the entire egg file
                 # p = p[:p.find('.egg') + 4]
@@ -155,7 +156,19 @@ def run_and_track_hadoop_job(arglist, tracking_url_callback=None, env=None):
     errors using those urls if the job fails. Throws HadoopJobError with information about the error (including stdout and stderr
     from the process) on failure and returns normally otherwise.
     '''
-    logger.info(' '.join(arglist))
+    logger.info('%s', ' '.join(arglist))
+
+    def write_luigi_history(arglist, history):
+        '''
+        Writes history to a file in the job's output directory in JSON format.
+        Currently just for tracking the job ID in a configuration where no history is stored in the output directory by Hadoop.
+        '''
+        history_filename = configuration.get_config().get('core', 'history-filename', '')
+        if history_filename:
+            output_dir = arglist[arglist.index('-output') + 1]
+            f = luigi.hdfs.HdfsTarget(os.path.join(output_dir, history_filename)).open('w')
+            f.write(json.dumps(history))
+            f.close()
 
     def track_process(arglist, tracking_url_callback, env=None):
         # Dump stdout to a temp file, poll stderr and log it
@@ -178,21 +191,22 @@ def run_and_track_hadoop_job(arglist, tracking_url_callback=None, env=None):
                     kill_job(job_id)
                 raise
             err_lines.append(err_line)
-            if err_line.strip():
-                logger.info(err_line.strip())
+            err_line = err_line.strip()
+            if err_line:
+                logger.info('%s', err_line)
             if err_line.find('Tracking URL') != -1:
-                tracking_url = err_line.strip().split('Tracking URL: ')[-1]
+                tracking_url = err_line.split('Tracking URL: ')[-1]
                 try:
                     tracking_url_callback(tracking_url)
                 except Exception as e:
-                    logger.error("Error in tracking_url_callback, disabling! %s" % e)
+                    logger.error("Error in tracking_url_callback, disabling! %s", e)
                     tracking_url_callback = lambda x: None
             if err_line.find('Running job') != -1:
                 # hadoop jar output
-                job_id = err_line.strip().split('Running job: ')[-1]
+                job_id = err_line.split('Running job: ')[-1]
             if err_line.find('submitted hadoop job:') != -1:
                 # scalding output
-                job_id = err_line.strip().split('submitted hadoop job: ')[-1]
+                job_id = err_line.split('submitted hadoop job: ')[-1]
 
         # Read the rest + stdout
         err = ''.join(err_lines + [err_line for err_line in proc.stderr])
@@ -200,6 +214,7 @@ def run_and_track_hadoop_job(arglist, tracking_url_callback=None, env=None):
         out = ''.join(temp_stdout.readlines())
 
         if proc.returncode == 0:
+            write_luigi_history(arglist, {'job_id': job_id})
             return
 
         # Try to fetch error logs if possible
@@ -240,7 +255,7 @@ def fetch_task_failures(tracking_url):
     import mechanize
     timeout = 3.0
     failures_url = tracking_url.replace('jobdetails.jsp', 'jobfailures.jsp') + '&cause=failed'
-    logger.debug('Fetching data from %s' % failures_url)
+    logger.debug('Fetching data from %s', failures_url)
     b = mechanize.Browser()
     b.open(failures_url, timeout=timeout)
     links = list(b.links(text_regex='Last 4KB'))  # For some reason text_regex='All' doesn't work... no idea why
@@ -254,7 +269,7 @@ def fetch_task_failures(tracking_url):
             r = b2.open(task_url, timeout=timeout)
             data = r.read()
         except Exception, e:
-            logger.debug('Error fetching data from %s: %s' % (task_url, e))
+            logger.debug('Error fetching data from %s: %s', task_url, e)
             continue
         # Try to get the hex-encoded traceback back from the output
         for exc in re.findall(r'luigi-exc-hex=[0-9a-f]+', data):
@@ -311,13 +326,13 @@ class HadoopJobRunner(JobRunner):
         output_tmp_fn = output_final + '-temp-' + datetime.datetime.now().isoformat().replace(':', '-')
         tmp_target = luigi.hdfs.HdfsTarget(output_tmp_fn, is_tmp=True)
 
-        arglist = ['hadoop', 'jar', self.streaming_jar]
+        arglist = [luigi.hdfs.load_hadoop_cmd(), 'jar', self.streaming_jar]
 
         # 'libjars' is a generic option, so place it first
         libjars = [libjar for libjar in self.libjars]
 
         for libjar in self.libjars_in_hdfs:
-            subprocess.call(['hadoop', 'fs', '-get', libjar, self.tmp_dir])
+            subprocess.call([luigi.hdfs.load_hadoop_cmd(), 'fs', '-get', libjar, self.tmp_dir])
             libjars.append(os.path.join(self.tmp_dir, os.path.basename(libjar)))
 
         if libjars:
@@ -346,9 +361,11 @@ class HadoopJobRunner(JobRunner):
 
         arglist += self.streaming_args
 
-        arglist += ['-mapper', map_cmd, '-reducer', red_cmd]
+        arglist += ['-mapper', map_cmd]
         if job.combiner != NotImplemented:
             arglist += ['-combiner', cmb_cmd]
+        if job.reducer != NotImplemented:
+            arglist += ['-reducer', red_cmd]
         files = [runner_path, self.tmp_dir + '/packages.tar', self.tmp_dir + '/job-instance.pickle']
 
         for f in files:
@@ -379,7 +396,7 @@ class HadoopJobRunner(JobRunner):
 
     def finish(self):
         if self.tmp_dir and os.path.exists(self.tmp_dir):
-            logger.debug('Removing directory %s' % self.tmp_dir)
+            logger.debug('Removing directory %s', self.tmp_dir)
             shutil.rmtree(self.tmp_dir)
 
     def __del__(self):
@@ -431,6 +448,13 @@ class LocalJobRunner(JobRunner):
 
         map_input.seek(0)
 
+        if job.reducer == NotImplemented:
+            # Map only job; no combiner, no reducer
+            map_output = job.output().open('w')
+            job._run_mapper(map_input, map_output)
+            map_output.close()
+            return
+
         # run job now...
         map_output = StringIO.StringIO()
         job._run_mapper(map_input, map_output)
@@ -460,13 +484,18 @@ class BaseHadoopJobTask(luigi.Task):
     final_combiner = NotImplemented
     final_reducer = NotImplemented
 
+    reducer = NotImplemented
+
     _counter_dict = {}
     task_id = None
 
     def jobconfs(self):
         jcs = []
         jcs.append('mapred.job.name=%s' % self.task_id)
-        jcs.append('mapred.reduce.tasks=%s' % self.n_reduce_tasks)
+        if self.reducer == NotImplemented:
+            jcs.append('mapred.reduce.tasks=0')
+        else:
+            jcs.append('mapred.reduce.tasks=%s' % self.n_reduce_tasks)
         pool = self.pool
         if pool is not None:
             # Supporting two schedulers: fair (default) and capacity using the same option
@@ -581,10 +610,6 @@ class JobTask(BaseHadoopJobTask):
 
     combiner = NotImplemented
 
-    def reducer(self, key, values):
-        for v in values:
-            yield key, v
-
     def incr_counter(self, *args, **kwargs):
         """ Increments a Hadoop counter
 
@@ -698,7 +723,10 @@ class JobTask(BaseHadoopJobTask):
         self.init_hadoop()
         self.init_mapper()
         outputs = self._map_input((line[:-1] for line in stdin))
-        self.internal_writer(outputs, stdout)
+        if self.reducer == NotImplemented:
+            self.writer(outputs, stdout)
+        else:
+            self.internal_writer(outputs, stdout)
 
     def _run_reducer(self, stdin=sys.stdin, stdout=sys.stdout):
         """Run the reducer on the hadoop node."""
