@@ -15,89 +15,47 @@
 import worker
 import lock
 import logging
+import logging.config
 import rpc
 import optparse
 import scheduler
 import warnings
-
-from ConfigParser import RawConfigParser, NoOptionError, NoSectionError
+import configuration
 import task
 import parameter
+import re
+import argparse
+import sys
+import os
+
 from task import Register
 
 
-def setup_interface_logging():
+def setup_interface_logging(conf_file=None):
     # use a variable in the function object to determine if it has run before
     if getattr(setup_interface_logging, "has_run", False):
         return
 
-    logger = logging.getLogger('luigi-interface')
-    logger.setLevel(logging.DEBUG)
+    if conf_file is None:
+        logger = logging.getLogger('luigi-interface')
+        logger.setLevel(logging.DEBUG)
 
-    streamHandler = logging.StreamHandler()
-    streamHandler.setLevel(logging.DEBUG)
+        streamHandler = logging.StreamHandler()
+        streamHandler.setLevel(logging.DEBUG)
 
-    formatter = logging.Formatter('%(levelname)s: %(message)s')
-    streamHandler.setFormatter(formatter)
+        formatter = logging.Formatter('%(levelname)s: %(message)s')
+        streamHandler.setFormatter(formatter)
+
+        logger.addHandler(streamHandler)
+    else:
+        logging.config.fileConfig(conf_file)
 
-    logger.addHandler(streamHandler)
     setup_interface_logging.has_run = True
 
 
-class LuigiConfigParser(RawConfigParser):
-    NO_DEFAULT = object()
-    _instance = None
-    _config_paths = ['/etc/luigi/client.cfg', 'client.cfg']
-
-    @classmethod
-    def add_config_path(cls, path):
-        cls._config_paths.append(path)
-        cls._instance.reload()
-
-    @classmethod
-    def instance(cls, *args, **kwargs):
-        """ Singleton getter """
-        if cls._instance is None:
-            cls._instance = cls(*args, **kwargs)
-            loaded = cls._instance.reload()
-            logging.getLogger('luigi-interface').info('Loaded %r' % loaded)
-
-        return cls._instance
-
-    def reload(self):
-        return self._instance.read(self._config_paths)
-
-    def _get_with_default(self, method, section, option, default, expected_type=None):
-        """ Gets the value of the section/option using method. Returns default if value
-        is not found. Raises an exception if the default value is not None and doesn't match
-        the expected_type.
-        """
-        try:
-            return method(self, section, option)
-        except (NoOptionError, NoSectionError):
-            if default is LuigiConfigParser.NO_DEFAULT:
-                raise
-            if expected_type is not None and default is not None and \
-               not isinstance(default, expected_type):
-                raise
-            return default
-
-    def get(self, section, option, default=NO_DEFAULT):
-        return self._get_with_default(RawConfigParser.get, section, option, default)
-
-    def getboolean(self, section, option, default=NO_DEFAULT):
-        return self._get_with_default(RawConfigParser.getboolean, section, option, default, bool)
-
-    def getint(self, section, option, default=NO_DEFAULT):
-        return self._get_with_default(RawConfigParser.getint, section, option, default, int)
-
-    def getfloat(self, section, option, default=NO_DEFAULT):
-        return self._get_with_default(RawConfigParser.getfloat, section, option, default, float)
-
-
 def get_config():
-    """ Convenience method (for backwards compatibility) for accessing config singleton """
-    return LuigiConfigParser.instance()
+    warnings.warn('Use luigi.configuration.get_config() instead')
+    return configuration.get_config()
 
 
 class EnvironmentParamsContainer(task.Task):
@@ -108,7 +66,7 @@ class EnvironmentParamsContainer(task.Task):
     # TODO(erikbern): would be cleaner if we don't have to read config in global scope
     local_scheduler = parameter.BooleanParameter(is_global=True, default=False,
                                                  description='Use local scheduling')
-    scheduler_host = parameter.Parameter(is_global=True, default=get_config().get('core', 'default-scheduler-host', default='localhost'),
+    scheduler_host = parameter.Parameter(is_global=True, default=configuration.get_config().get('core', 'default-scheduler-host', default='localhost'),
                                          description='Hostname of machine running remote scheduler')
     scheduler_port = parameter.IntParameter(is_global=True, default=8082,
                                             description='Port of remote scheduler api process')
@@ -143,36 +101,95 @@ def reset():
     warnings.warn('reset is no longer supported')
 
 
+class WorkerSchedulerFactory(object):
+    def create_local_scheduler(self):
+        return scheduler.CentralPlannerScheduler()
+
+    def create_remote_scheduler(self, host, port):
+        return rpc.RemoteScheduler(host=host, port=port)
+
+    def create_worker(self, scheduler, worker_processes):
+        return worker.Worker(scheduler=scheduler, worker_processes=worker_processes)
+
+
 class Interface(object):
     def parse(self):
         raise NotImplementedError
 
     @staticmethod
-    def run(tasks, override_defaults={}):
-        env_params = EnvironmentParamsContainer.env_params(override_defaults)
+    def run(tasks, worker_scheduler_factory=None, override_defaults={}):
 
-        if env_params.lock:
-            lock.run_once(env_params.lock_pid_dir)
+        if worker_scheduler_factory is None:
+            worker_scheduler_factory = WorkerSchedulerFactory()
+
+        env_params = EnvironmentParamsContainer.env_params(override_defaults)
+        logging_conf = configuration.get_config().get('core', 'logging_conf_file', None)
+        if logging_conf is not None and not os.path.exists(logging_conf):
+            raise Exception("Error: Unable to locate specified logging configuration file!")
+
+        if not configuration.get_config().getboolean('core', 'no_configure_logging', False):
+            setup_interface_logging(logging_conf)
+
+        if env_params.lock and not(lock.acquire_for(env_params.lock_pid_dir)):
+            sys.exit(1)
 
         if env_params.local_scheduler:
-            sch = scheduler.CentralPlannerScheduler()
+            sch = worker_scheduler_factory.create_local_scheduler()
         else:
-            sch = rpc.RemoteScheduler(host=env_params.scheduler_host, port=env_params.scheduler_port)
+            sch = worker_scheduler_factory.create_remote_scheduler(host=env_params.scheduler_host, port=env_params.scheduler_port)
+
+        w = worker_scheduler_factory.create_worker(scheduler=sch, worker_processes=env_params.workers)
 
-        w = worker.Worker(scheduler=sch, worker_processes=env_params.workers)
         for task in tasks:
             w.add(task)
         logger = logging.getLogger('luigi-interface')
         logger.info('Done scheduling tasks')
         w.run()
+        w.stop()
+
+
+class ErrorWrappedArgumentParser(argparse.ArgumentParser):
+    ''' Wraps ArgumentParser's error message to suggested similar tasks
+    '''
+
+    # Simple unweighted Levenshtein distance
+    def _editdistance(self, a, b):
+        r0 = range(0, len(b) + 1)
+        r1 = [0] * (len(b) + 1)
+
+        for i in range(0, len(a)):
+            r1[0] = i + 1
+
+            for j in range(0, len(b)):
+                c = 0 if a[i] is b[j] else 1
+                r1[j + 1] = min(r1[j] + 1, r0[j + 1] + 1, r0[j] + c)
+
+            r0 = r1[:]
+
+        return r1[len(b)]
+
+    def error(self, message):
+        result = re.match("argument .+: invalid choice: '(\w+)'.+", message)
+        if result:
+            arg = result.group(1)
+            weightedTasks = [(self._editdistance(arg, task), task) for task in Register.get_reg().keys()]
+            orderedTasks = sorted(weightedTasks, key=lambda pair: pair[0])
+            candidates = [task for (dist, task) in orderedTasks if dist <= 5 and dist < len(task)]
+            displaystring = ""
+            if candidates:
+                displaystring = "No task %s. Did you mean:\n%s" % (arg, '\n'.join(candidates))
+            else:
+                displaystring = "No task %s." % arg
+            super(ErrorWrappedArgumentParser, self).error(displaystring)
+        else:
+            super(ErrorWrappedArgumentParser, self).error(message)
 
 
 class ArgParseInterface(Interface):
     ''' Takes the task as the command, with parameters specific to it
     '''
     def parse(self, cmdline_args=None, main_task_cls=None):
-        import argparse
-        parser = argparse.ArgumentParser()
+        parser = ErrorWrappedArgumentParser()
 
         def _add_parameter(parser, param_name, param, prefix=None):
             description = []
@@ -207,7 +224,8 @@ class ArgParseInterface(Interface):
             _add_task_parameters(parser, main_task_cls)
 
         else:
-            subparsers = parser.add_subparsers(dest='command')
+            orderedtasks = '{%s}' % ','.join(sorted(Register.get_reg().keys()))
+            subparsers = parser.add_subparsers(dest='command', metavar=orderedtasks)
 
             for name, cls in Register.get_reg().iteritems():
                 subparser = subparsers.add_parser(name)
@@ -336,28 +354,31 @@ class OptParseInterface(Interface):
         return [task]
 
 
-def run(cmdline_args=None, existing_optparse=None, use_optparse=False, main_task_cls=None):
+class LuigiConfigParser(configuration.LuigiConfigParser):
+    ''' Deprecated class, use configuration.LuigiConfigParser instead. Left for backwards compatibility '''
+    pass
+
+
+def run(cmdline_args=None, existing_optparse=None, use_optparse=False, main_task_cls=None, worker_scheduler_factory=None):
     ''' Run from cmdline.
 
     The default parser uses argparse.
-    However for legacy reasons we support optparse that optinally allows for
+    However for legacy reasons we support optparse that optionally allows for
     overriding an existing option parser with new args.
     '''
-    setup_interface_logging()
     if use_optparse:
         interface = OptParseInterface(existing_optparse)
     else:
         interface = ArgParseInterface()
     tasks = interface.parse(cmdline_args, main_task_cls=main_task_cls)
-    interface.run(tasks)
+    interface.run(tasks, worker_scheduler_factory)
 
 
-def build(tasks, **env_params):
+def build(tasks, worker_scheduler_factory=None, **env_params):
     ''' Run internally, bypassing the cmdline parsing.
 
     Useful if you have some luigi code that you want to run internally.
     Example
     luigi.build([MyTask1(), MyTask2()], local_scheduler=True)
     '''
-    setup_interface_logging()
-    Interface.run(tasks, env_params)
+    Interface.run(tasks, worker_scheduler_factory, env_params)
